// $Id$

//=============================================================================
/**
 *  @file    command.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_COMMAND_H
#define JYUGEM_BEE_COMMAND_H

#include "jyugem/bee.h"
#include "jyugem/bee/parameter.h"
#include "jyugem/bee/sequential_map.h"

namespace jyugem { namespace bee
{

//
// Value class
//
class BEE_Export command : public nameable
{
public:
    typedef sequential_map<parameter, std::string, &parameter::name >
        parameter_list;

    command(const string& name) : nameable(name) {}
    command(const command& rhs) { this->copy(rhs); }
    ~command()
    {
        for (size_t i = 0; i < m_parameters.size(); i++)
        {
            if (m_parameters[i] != null)
                delete m_parameters[i];
        }
    }

    command& operator=(const command& rhs)
    {
        if (this == &rhs)
            return *this;
        this->copy(rhs);
        return *this;
    }

    const std::string& name() { return this->uniqueName(); }
    const std::string& name() const { return this->uniqueName(); }

    parameter * find(const std::string& pname)
    { return m_parameters.find(pname); }

    int add(const parameter& param)
    { return m_parameters.add(new parameter(param)); }

protected:
    command() : nameable(_TX("")) {}
    void copy(const command& rhs)
    {
        this->nameable::operator=(rhs);
        for (size_t i = 0; i < rhs.m_parameters.size(); i++)
        {
            parameter * src = rhs.m_parameters[i];
            m_parameters.add(new parameter(*src));
        }
    }

protected:
    parameter_list m_parameters;
};


} } // *namespace*

#endif  /* JYUGEM_BEE_COMMAND_H */
