/* $Id$ */

//=============================================================================
/**
 *  @file    trace.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUBEM_BEE_TRACE_H
#define JYUBEM_BEE_TRACE_H

#include "jyugem/bee/beeconf.h"
#include "jyugem/bee/ace_wrappers.h"

namespace jyugem { namespace bee
{
//
//
//
class BEE_Export trace
{
public:
  enum
  {
    LV_MUST  = 0,
    LV_HIGH,
    LV_VIEW,
    LV_MIDDLE,
    LV_LOW,
    LV_CONSTRUCT,
    LV_IGNORE,
  };

  trace(const BCHAR * fname, int l = LV_LOW)
    : m_function(fname), m_file(null), m_linenum(0)
  {
    if (l >= level())
    {
      ACE_DEBUG((LM_DEBUG, ACE_TEXT("%s\n"), m_function));
    }
  }
  trace(const BCHAR * funcname, const BCHAR * filename, int lnum)
    : m_function(funcname), m_file(filename), m_linenum(lnum)
  {
    //ACE_DEBUG((LM_DEBUG,
    //    ACE_TEXT("%s(%d):%s\n"), m_file, m_linenum, m_function));
  }
  ~trace() {}

  const BCHAR * function_name() { return m_function; }

  // Trace error message
  void trace::error(const BCHAR * fmt, ...)
  {
    va_list arglist;
    va_start(arglist, fmt);

    ACE_ERROR((LM_ERROR, ACE_TEXT("%T %s: "), m_function));
    ACE_ERROR((fmt, LM_ERROR, arglist));

    va_end(arglist);
  }

  // Trace degug message
  void trace::debug(const BCHAR * fmt, ...)
  {
    va_list arglist;
    va_start(arglist, fmt);

    ACE_DEBUG((LM_DEBUG, ACE_TEXT("%T %s: "), m_function));
    ACE_DEBUG((fmt, LM_DEBUG, arglist));

    va_end(arglist);
  }

  // Trace info message
  void trace::notice(const BCHAR * fmt, ...)
  {
    va_list arglist;
    va_start(arglist, fmt);

    ACE_ERROR((LM_NOTICE, ACE_TEXT("%T %s: "), m_function));
    ACE_ERROR((fmt, LM_NOTICE, arglist));

    va_end(arglist);
  }

  static void level(int level);
  static int  level();

private:
  trace() : m_function(null), m_file(null), m_linenum(0) {}

  const BCHAR * m_function;
  const BCHAR * m_file;
  int   m_linenum;
};

} } // *namespace* jyugem::bee

#endif  // JYUGEM_BEE_TRACE_H
