// $Id$

//=============================================================================
/**
 *  @file  variable_def.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_VARIABLE_DEF_H
#define JYUGEM_BEE_VARIABLE_DEF_H

#include "jyugem/bee.h"

namespace jyugem { namespace bee
{

//
// Interface class
//
class BEE_Export variable_def : public parameter
{
public:
  variable_def(const std::string& name, const bee::value& init)
    : parameter(name, init) {}
  variable_def(const std::string& name, const bee::value& init,
         const bee::value& minval, const bee::value& maxval,
         const std::string& unit)
    : parameter(name), m_init(init), m_min(minval), m_max(maxval),
      m_unit(unit) {}
  variable_def(const parameter& rhs) { this->copy(rhs); }
  virtual ~variable_def() {}

  variable_def& operator=(const variable_def& rhs)
  {
    if (this == &rhs)
      return *this;
    this->copy(rhs);
    return *this;
  }

  virtual const bee::value& init() const { return m_value; }
  virtual const bee::value& min() const  { return m_min; }
  virtual const bee::value& max() const  { return m_max; }
  virtual const std::string& unit() const { return m_unit; }
  virtual bool  effective(const bee::value& v)
  {
    if (! m_min.nil() && m_min > v)
    {
      return false;
    }
    if (! m_min.nil() && v > m_max)
    {
      return false;
    }
    return true;
  }

  virtual variable * create() const
  {
    return (new variable(m_vardef->uniqueName(), m_vardef->init()));
  }
  virtual variable * create(const std::string& baseName) const
  {
    std::string varName(baseName);
    varName += _TX("/");
    varName += m_vardef->uniqueName();
    return (new variable(varName(), m_vardef->init()));
  }

//
protected:
  variable_def() : parameter(_TX("")) {}
  void copy(const variable_def& rhs)
  {
    this->bee::parameter::operator=(rhs);
    m_min = rhs.m_min;
    m_max = rhs.m_max;
    m_unit = rhs.m_unit;
  }

protected:
  bee::value m_min;
  bee::value m_max;
  std::string m_unit;
};


} } // *namespace*

#endif
