// $Id$

//=============================================================================
/**
 *  @file    LoadPort.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_CMS_LOADPORT_H
#define JYUGEM_CMS_LOADPORT_H

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif


#include <boost/mpl/list.hpp>
#include <boost/format.hpp>

#include "jyugem/bee/state_task.h"
namespace bee = jyugem::bee;


//
namespace jyugem { namespace cms
{

class Carrier;
typedef boost::shared_ptr<Carrier> CarrierPtr;

//-----------------------------------------------------------------------------
//
// E87 : SPECIFICATION FOR CARRIER MANAGEMENT
//
//       LoadPort class
//
//-----------------------------------------------------------------------------
class LoadPort : public bee::state_task
{
  friend class LoadPortState;

public:
  LoadPort(bee::byte portid = 0);
  // LoadPort(const LoadPort& carrier);
  virtual ~LoadPort();

  // LoadPort& operator=(const LoadPort& rhs)
  // {
  //   if ( this== &rhs)
  //     return *this;
  //   this->copy(rhs);
  //   return *this;
  // }

  // Outhgonal region of state
  enum Orthogonal { TRANSFER = 0, ACCESS = 1, RESERVATION = 2, ASSOCIATION = 3 };

  // Access Mode
  enum { HOST_ONLY, EQUIPMENT_ONLY, BOTH_ACCESS };

  // Undock/Unclamp Style
  enum { CARRIER_COMPLETE, MOVED_OUT };

  // Manual In/Out Timer
  enum { SWITCH_SWITCH, SWITCH_TIMER, NOP_TIMER, NOP_SWITCH };

  int open0();

  const BCHAR * current_state(Orthogonal region = Orthogonal::TRANSFER);

  CarrierPtr carrier() { return m_carrier; }
  int onCarrier();
  int offCarrier();

protected:
//  void copy(const LoadPort& rhs);
//  void setCarrier(const CarrierPtr& carrier) { m_carrier = carrier; }

public:
  bool m_bypassReadID;

  // Status variables
  int  m_portTransferState;

  // Constant variables
  int m_accessMode;
  int m_manualTimer;
  int m_manualInType;
  int m_manualOutType;
  int m_undockStyle;
  int m_unclampStyle;

  //
  CarrierPtr m_carrier;
  bee::byte m_reservationState;
  bee::byte m_portAssosiationState;
  bee::byte m_portID;
  bee::byte m_reason;
};


} }  // *namespace* jyugem::cms

#endif
