// $Id$

//=============================================================================
/**
 *  @file    metaplural_t.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include <boost/lexical_cast.hpp>
#include <boost/tokenizer.hpp>

#include "jyugem/bee.h"
#include "meta_template.h"
using namespace jyugem::bee;

namespace jyugem { namespace bee
{

template<typename T>
bool metaplural_t<T>::equal(const bee::value& lhs, const bee::value& rhs)
{
  TRACE_FUNCTION();
  try
  {
    T * lptr = const_cast<bee::value&>(lhs).any_cast<T>();
    T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
    T::const_iterator li = lptr->begin();
    T::const_iterator ri = rptr->begin();
    for ( ; li != lptr->end() && ri != rptr->end(); ++li, ++ri)
    {
      if (*li != *ri)
      {
        return false;
      }
    }

    if (li == lptr->end() && ri == rptr->end())
    {
      return true;
    }
    return false;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<typename T>
bool metaplural_t<T>::less(const bee::value& lhs, const bee::value& rhs)
{
  TRACE_FUNCTION();
  try
  {
    T * lptr = const_cast<bee::value&>(lhs).any_cast<T>();
    T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
    T::iterator li = lptr->begin();
    T::iterator ri = rptr->begin();
    for ( ; li != lptr->end() && ri != rptr->end(); ++li, ++ri)
    {
      if (*li > *ri)
      {
        return false;
      }
      else if (*li < *ri)
      {
        return true;
      }
    }

    if (li == lptr->end() && ri == rptr->end())
    {
      return false;
    }
    else if (li == lptr->end())
    {
      return true;
    }
    return false;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<typename T>
bool metaplural_t<T>::getBool(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    T::const_iterator i = vptr->begin();
    for ( ; i != vptr->end(); ++i)
    {
      if (*i == 0)
      {
        return false;
      }
    }
    return true;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<typename T>
int metaplural_t<T>::getInt(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    return (int)(*vptr)[0];
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

template<typename T>
UINT metaplural_t<T>::getUInt(const value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    return (UINT)(*vptr)[0];
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return 0;
}

template<typename T>
int metaplural_t<T>::setInt(bee::value& v, int val)
{
  TRACE_FUNCTION();
  typedef typename T::value_type value_type;
  value_type data = (value_type)val;
  try
  {
    T * vptr = v.any_cast<T>();
    if (vptr->empty())
    {
      vptr->push_back(data);
    }
    else
    {
      (*vptr)[0] = data;
    }
    return 0;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

template<typename T>
int metaplural_t<T>::set(bee::value& v, const value& from)
{
  TRACE_FUNCTION();
  typedef typename T::value_type value_type;
  if (v.type() == from.type())
  {
    try
    {
      T * lptr = v.any_cast<T>();
      T * rptr = const_cast<bee::value&>(from).any_cast<T>();

      lptr->clear();  // comment out if append data.
      std::copy(rptr->begin(), rptr->end(), lptr->begin());
      return 0;
    }
    catch (const boost::bad_any_cast& )
    {
      TRACE_ERROR((_TX("bad_any_cast \n")));
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Type is different(%d & %d)\n"),
                 v.type(), from.type()));
  }
  return -1;
}

template<typename T>
int metaplural_t<T>::set(bee::value& v, const string& valstr)
{
  TRACE_FUNCTION();
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
  try
  {
    T * vvptr = v.any_cast<T>();
    vvptr->clear();  // comment out if append data.

    boost::char_separator<char> sep(",");
    tokenizer tokens(valstr, sep);
    for (tokenizer::iterator iter = tokens.begin();
      iter != tokens.end(); ++iter)
    {
      T::value_type newval = 
          boost::lexical_cast<T::value_type>(*iter);
      vvptr->push_back(newval);
    }
    return 0;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  catch(const boost::bad_lexical_cast& lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
  }
  return -1;
}

template<typename T>
int metaplural_t<T>::set(bee::value& v, const BYTE * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null)
    return -1;

  typedef typename T::value_type value_type;
  T * vvptr;
  try
  {
    vvptr = v.any_cast<T>();
    vvptr->clear();  // comment out if append data.
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
    return -1;
  }

  if (order == ENDIAN || sizeof(value_type) == 1)
  {
    value_type * ptr = (value_type *)bin;
    size_t dc = q / sizeof(value_type);
    for (size_t i = 0; i < dc; i++)
    {
      vvptr->push_back(*ptr);
      ptr++;
    }
  }
  else if (sizeof(value_type) == 2)
  {
    size_t dc = q / sizeof(value_type);
    BYTE * rptr = const_cast<BYTE *>(bin);
    for (size_t i = 0; i < dc; i++)
    {
      value_type newval  = (value_type)(*rptr << 8) + (*(rptr + 1));
      vvptr->push_back(newval);
      rptr++;
    }
  }
  else if (sizeof(value_type) == 4)
  {
    size_t dc = q / sizeof(value_type);
    BYTE * rptr = const_cast<BYTE *>(bin);
    for (size_t i = 0; i < dc; i++)
    {
      value_type newval;
      BYTE * bufptr = (BYTE *)&newval;
      *(bufptr + 3) = *(rptr);
      *(bufptr + 2) = *(rptr + 1);
      *(bufptr + 1) = *(rptr + 2);
      *(bufptr)     = *(rptr + 3);
      vvptr->push_back(newval);
      rptr++;
    }
  }
  else if (sizeof(value_type) == 8)
  {
    size_t dc = q / sizeof(value_type);
    BYTE * rptr = const_cast<BYTE *>(bin);
    for (size_t i = 0; i < dc; i++)
    {
      value_type newval;
      BYTE * bufptr = (BYTE *)&newval;
      *(bufptr + 7) = *(rptr);
      *(bufptr + 6) = *(rptr + 1);
      *(bufptr + 5) = *(rptr + 2);
      *(bufptr + 4) = *(rptr + 3);
      *(bufptr + 3) = *(rptr + 4);
      *(bufptr + 2) = *(rptr + 5);
      *(bufptr + 1) = *(rptr + 6);
      *(bufptr)     = *(rptr + 7);
      vvptr->push_back(newval);
      rptr++;
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Illegal value size(%d)\n"), sizeof(value_type)));
    return -1;
  }
  return 0;
}


template<typename T>
int metaplural_t<T>::get(const bee::value& v, BYTE * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null)
    return -1;

  typedef typename T::value_type value_type;
  T * vvptr;
  try
  {
    vvptr = const_cast<bee::value&>(v).any_cast<T>();
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
    return -1;
  }

  BYTE * dataptr = bin;
  value_type curval;
  BYTE * vptr = (BYTE *)&curval;
  size_t dc = MIN(vvptr->size(), q / sizeof(value_type));
  if (order == ENDIAN || sizeof(value_type) == 1)
  {
    value_type * valptr = (value_type *)bin;
    for (size_t i = 0; i < dc; ++i)
    {
      *valptr = (*vvptr)[i];
      valptr++;
    }
  }
  else if (sizeof(value_type) == 2)
  {
    for (size_t i = 0; i < dc; ++i)
    {
      curval = (*vvptr)[i];
      *(dataptr + 1) = *(vptr);
      *(dataptr)     = *(vptr + 1);
    }
  }
  else if (sizeof(value_type) == 4)
  {
    for (size_t i = 0; i < dc; ++i)
    {
      curval = (*vvptr)[i];
      *(dataptr + 3) = *(vptr);
      *(dataptr + 2) = *(vptr + 1);
      *(dataptr + 1) = *(vptr + 2);
      *(dataptr)     = *(vptr + 3);
    }
  }
  else if (sizeof(value_type) == 8)
  {
    for (size_t i = 0; i < dc; ++i)
    {
      curval = (*vvptr)[i];
      *(dataptr + 7) = *(vptr);
      *(dataptr + 6) = *(vptr + 1);
      *(dataptr + 5) = *(vptr + 2);
      *(dataptr + 4) = *(vptr + 3);
      *(dataptr + 3) = *(vptr + 4);
      *(dataptr + 2) = *(vptr + 5);
      *(dataptr + 1) = *(vptr + 6);
      *(dataptr)     = *(vptr + 7);
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Illegal value size(%d)\n"), sizeof(T)));
    return -1;
  }
  return (dc * sizeof(value_type));
}

template<typename T>
int metaplural_t<T>::get(const bee::value& v, std::string& retstr)
{
  TRACE_FUNCTION();
  typedef typename T::value_type value_type;
  try
  {
    T * vvptr = const_cast<bee::value&>(v).any_cast<T>();
    retstr.clear();
    for (size_t i = 0; i < vvptr->size(); i++)
    {
      value_type val = (*vvptr)[i];
      retstr += boost::lexical_cast<std::string>(val);
    }
    return 0;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  catch (const boost::bad_lexical_cast & lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
  }
  return -1;
}

template<typename T>
void metaplural_t<T>::copy(bee::value& v, const bee::value& rhs)
{
  TRACE_FUNCTION();
  typedef typename T::value_type value_type;
  if (v.type() == rhs.type())
  {
    try
    {
      T * lptr = v.any_cast<T>();
      T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();

      lptr->clear();  // comment out if append data.
      std::copy(rptr->begin(), rptr->end(), lptr->begin());
      return ;
    }
    catch (const boost::bad_any_cast& )
    {
      TRACE_ERROR((_TX("bad_any_cast \n")));
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Type is different(%d & %d)\n"),
                 v.type(), rhs.type()));
  }
  return ;
}

// bytes in list
template<typename T>
size_t metaplural_t<T>::length(const bee::value& v) const
{
  TRACE_FUNCTION();
  typedef typename T::value_type value_type;
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    return (vptr->size() * sizeof(value_type));
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return 0;
}

template<typename T>
int jyugem::bee::metaplural_t<T>::make(value& outv)
{
  T seed;
  outv = seed;
  return 0;
}

} }  // *namespace* - jyugem::bee
