// $Id$

//=============================================================================
/**
 *  @file  metavalue_t.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include <boost/lexical_cast.hpp>
#include "jyugem/bee.h"
#include "meta_template.h"
using namespace jyugem::bee;

namespace jyugem { namespace bee
{

template<class T>
bool metavalue_t<T>::equal(const bee::value& lhs,
  const bee::value& rhs)
{
  TRACE_FUNCTION();
  try
  {
    T * lptr = const_cast<bee::value&>(lhs).any_cast<T>();
    T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
    many::iterator li = lptr->begin();
    many::iterator ri = rptr->begin();
    for ( ; li != lptr->end() && ri != rptr->end(); ++li, ++ri)
    {
      const value& lv = *li;
      const value& rv = *ri;
      if (lv != rv)
      {
        return false;
      }
    }

    if (li == lptr->end() && ri == rptr->end())
    {
      return true;
    }
    return false;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<class T>
bool metavalue_t<T>::less(const bee::value& lhs,
  const bee::value& rhs)
{
  TRACE_FUNCTION();
  try
  {
    T * lptr = const_cast<bee::value&>(lhs).any_cast<T>();
    T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
    many::iterator li = lptr->begin();
    many::iterator ri = rptr->begin();
    for ( ; li != lptr->end() && ri != rptr->end(); ++li, ++ri)
    {
      const value& lv = *li;
      const value& rv = *ri;
      if (lv < rv)
      {
        return true;
      }
      else if (! (lv == rv))
      { // (lv > rv)
        return false;
      }
    }

    if (li == lptr->end() && ri == rptr->end())
    {
      return false;
    }
    else if (li == lptr->end())
    {
      return true;
    }
    return false;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}


template<class T>
bool metavalue_t<T>::getBool(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    many::iterator iter = varray->begin();
    for ( ; iter != varray->end(); ++iter)
    {
      if (! (*iter).getBool())
      {
        return false;
      }
    }
    return true;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<class T>
int metavalue_t<T>::getInt(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    return varray->front().getInt();
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

template<class T>
UINT metavalue_t<T>::getUInt(const value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    return varray->front().getUInt();
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return 0;
}

template<class T>
int metavalue_t<T>::setInt(bee::value& v, int val)
{
  TRACE_FUNCTION();
  try
  {
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    return varray->front().setInt(val);
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

template<class T>
int metavalue_t<T>::set(bee::value& v, const value& from)
{
  TRACE_FUNCTION();
  if (v.type() == from.type())
  {
    try
    {
      T * lptr = v.any_cast<T>();
      T * rptr = const_cast<bee::value&>(from).any_cast<T>();
      *lptr = *rptr;
    }
    catch (const boost::bad_any_cast& )
    {
      TRACE_ERROR((_TX("bad_any_cast \n")));
      return -1;
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Type is different(%d & %d)\n"),
          v.type(), from.type()));
    return -1;
  }
  return 0;
}

template<class T>
int metavalue_t<T>::set(bee::value& v, const string& valstr)
{
  TRACE_FUNCTION();
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
  try
  {
    T * vvptr = v.any_cast<T>();
    vvptr->clear();  // comment out if append data.

    boost::char_separator<char> sep1("{}");
    tokenizer tokens1(valstr, sep1);
    for (tokenizer::iterator iter1 = tokens1.begin();
       iter1 != tokens1.end(); ++iter1)
    {
      boost::char_separator<char> sep2(",");
      tokenizer tokens2(*iter1, sep2);
      for (tokenizer::iterator iter2 = tokens2.begin();
         iter2 != tokens2.end(); ++iter2)
      {
        vvptr->push_back(value(*iter2));
      }
    }
    return 0;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

template<class T>
int metavalue_t<T>::set(bee::value& v, const BYTE * bin,
  size_t q, int order)
{
  TRACE_FAIL((_TX("Not support\n")));
  return -1;
}

template<class T>
int metavalue_t<T>::get(const bee::value& v, BYTE * bin,
  size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null)
    return -1;

  try
  {
    BYTE * curptr = bin;
    int total = 0;
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    many::iterator iter = varray->begin();
    for ( ; iter != varray->end(); ++iter)
    {
      if (q <= (size_t)total)
      {
        return total;
      }
      int bytes = (*iter).get(bin, q, order);
      if (bytes < 0)
      {
        return -1;
      }
      total += bytes;
    }
    return total;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

template<class T>
int metavalue_t<T>::get(const bee::value& v, std::string& retstr)
{
  TRACE_FUNCTION();
  try
  {
    retstr.clear();
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    many::iterator iter = varray->begin();
    for ( ; iter != varray->end(); ++iter)
    {
      std::string curstr;
      int result = (*iter).get(curstr);
      if (result < 0)
      {
        return -1;
      }
      retstr += curstr;
    }
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
    return -1;
  }
  return 0;
}

template<class T>
void metavalue_t<T>::copy(bee::value& v, const bee::value& rhs)
{
  TRACE_FUNCTION();
  if (v.type() == rhs.type())
  {
    TRACE_ERROR((_TX("Different value type\n")));
    return ;
  }

  try
  {
    T * larray = v.any_cast<T>();
    T * rarray = const_cast<bee::value&>(rhs).any_cast<T>();
    *larray = *rarray;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return ;
}

// bytes in list
template<class T>
size_t metavalue_t<T>::length(const bee::value& v) const
{
  TRACE_FUNCTION();
  size_t total = 0;
  try
  {
    T * varray = const_cast<bee::value&>(v).any_cast<T>();
    many::iterator iter = varray->begin();
    for ( ; iter != varray->end(); ++iter)
    {
      total += (*iter).length();
    }
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
    return 0;
  }
  return total;
}


template<class T>
int metavalue_t<T>::make(value& outv)
{
  T seed;
  outv = seed;
  return 0;
}

} }  // *namespace* - jyugem::bee
