// $Id$

//=============================================================================
/**
 *  @file  value.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee.h"
#include "jyugem/bee/metadata.h"
using namespace jyugem::bee;

namespace jyugem { namespace bee
{

value value::nil;

//
// value class
//
value::value(const value& rhs)
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(rhs.m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    throw failure(_TX("Illegal type code"));
  }
  meta->make(*this);
  meta->copy(*this, rhs);
}

value& value::operator=(const value& rhs)
{
  TRACE_FUNCTION();
  if (this == &rhs)
    return *this;
  metadata * meta = metadata::instance(rhs.m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return *this;
  }
  meta->make(*this);
  meta->copy(*this, rhs);
  return *this;
}

size_t value::size() const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return 0;
  }
  return meta->size();
}

size_t value::length() const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return 0;
  }
  return meta->length(*this);
}

//
//
// Get and Set concrete type
//
bool value::getBool() const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return false;
  }
  return meta->getBool(*this);
}

int value::getInt() const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->getInt(*this);
}

UINT value::getUInt() const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return 0;
  }
  return meta->getUInt(*this);
}

int value::setInt(int v)
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->setInt(*this, v);
}

//
// Copy operator
//
int value::set(const value& val)
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->set(*this, val);
}

int value::set(unsigned char * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->set(*this, bin, q, order);
}

// Set to value
int value::set(const string& valstr)
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->set(*this, valstr);
}

int value::get(unsigned char * buf, size_t size, int order) const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->get(*this, buf, size, order);
}

int value::get(string& retval) const
{
  TRACE_FUNCTION();
  metadata * meta = metadata::instance(m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), m_t));
    return -1;
  }
  return meta->get(*this, retval);
}

bool value::add(const value& v)
{
  TRACE_FUNCTION();
  bool result = false;
  if (m_t == BEE_ARRAY)
  {
    try
    {
      array_val * vals = boost::any_cast<array_val>(&m_any);
      vals->push_back(v);
      result = true;
    }
    catch (const boost::bad_any_cast &)
    {
      TRACE_ERROR((_TX("Any can't cast array.\n")));
    }
  }
  else if (m_t == BEE_STRUCT)
  {
    try
    {
      struct_val * tb = boost::any_cast<struct_val>(&m_any);
      tb->push_back(v);
      result = true;
    }
    catch (const boost::bad_any_cast &)
    {
      TRACE_ERROR((_TX("Any can't cast struct.\n")));
    }
  }
  else
  {
    TRACE_ERROR((_TX("Type is neather array nor struct(%d).\n"), m_t));
  }
  return result;
}


//
// calculator
//
bool operator==(const value& lhs, const value& rhs)
{
  TRACE_FUNCTION();
  if (lhs.m_t != rhs.m_t)
  {
    TRACE_ERROR((_TX("Difference type code(%d & %d).\n"), lhs.m_t, rhs.m_t));
    return false;
  }
  metadata * meta = metadata::instance(lhs.m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), lhs.m_t));
    return false;
  }
  return meta->equal(lhs, rhs);
}

bool operator<(const value& lhs, const value& rhs)
{
  TRACE_FUNCTION();
  if (lhs.m_t != rhs.m_t)
  {
    TRACE_ERROR((_TX("Difference type code(%d & %d).\n"), lhs.m_t, rhs.m_t));
    return false;
  }
  metadata * meta = metadata::instance(lhs.m_t);
  if (meta == null)
  {
    TRACE_ERROR((_TX("Illegal type code(%d).\n"), lhs.m_t));
    return false;
  }
  return meta->less(lhs, rhs);
}


} }  // *namespace* - jyugem::bee

