// $Id$

//=============================================================================
/**
 *  @file    cidrw.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif

#include "state/CIDRWState.h"
#include "jyugem/cidrw/cidrw.h"
#include "jyugem/gem/event_manager.h"

namespace bee = jyugem::bee;
namespace gem = jyugem::gem;

//
namespace jyugem { namespace cidrw
{

//
// Carrier ID Reader/Writer
//

CIDRW::CIDRW(int hc) : bee::state_task("CIDRW"),
  m_alarmStatus(NO_ALRAMS), m_operationalStatus(IDLE), m_installed(),
  m_Configuration(_TX("")), m_SoftwareRevisionLevel(_TX("")),
  m_DeviceType(_TX("Reader")), m_HardwareRevisionLevel(_TX("")),
  m_MaintenanceData(_TX("")), m_Manufacturer(_TX("")), m_ModelNumber(_TX("")),
  m_SerialNumber(_TX(""))
{
  CIDRWState * rws = new CIDRWState(this);
  m_status = rws;

  for (int i = 0; i < hc; i++)
  {
    BCHAR buf[32];
    _stprintf(buf, _TX("%02d"), i);
    m_heads.push_back(new CIDRWHead(buf));
  }
  
}

CIDRW::~CIDRW()
{
  for (size_t i = 0; i < m_heads.size(); i++)
  {
    delete m_heads[i];
  }

  CIDRWState * rws = static_cast<CIDRWState *>(m_status);
  rws->terminate();
  delete rws;
  
}

#if 0
void CIDRW::copy(const CIDRW& rhs)
{
  this->bee::state_task::operator =(rhs);

  m_alarmStatus = rhs.m_alarmStatus;
  m_operationalStatus = rhs.m_operationalStatus;
  m_installed = rhs.m_installed;

  m_Configuration = rhs.m_Configuration;
  m_SoftwareRevisionLevel = rhs.m_SoftwareRevisionLevel;
  m_DeviceType = rhs.m_DeviceType;
  m_HardwareRevisionLevel = rhs.m_HardwareRevisionLevel;
  m_MaintenanceData = rhs.m_MaintenanceData;
  m_Manufacturer = rhs.m_Manufacturer;
  m_ModelNumber = rhs.m_ModelNumber;
  m_SerialNumber = rhs.m_SerialNumber;
}
#endif

gem::event_manager evtmngr;

int CIDRW::open0()
{
  ace::init();

  int result = this->open(null);
  this->on_trigger_connect(boost::bind(&gem::event_manager::notify, &evtmngr, _1, _2));

  CIDRWState * rws = static_cast<CIDRWState *>(m_status);
  rws->initiate();

  for (size_t i = 0; i < m_heads.size(); i++)
  {
    m_heads[i]->open0();
  }

  return result;
}

const BCHAR * CIDRW::current_state()
{
  CIDRWState * cidrws = static_cast<CIDRWState *>(m_status);
  const cstr_id * stp = cidrws->get_current_name();
  if (stp)
    return stp->name();
  return null;
}

const BCHAR * CIDRW::head_state(int hid)
{
  if ((hid <= 0) && (hid > int(m_heads.size() + 1)))
  {
    return _TX("Illegal head ID.");
  }
  return m_heads[hid - 1]->current_state();
}

const int CIDRW::update_head(int hid, const std::string& evtname)
{
  if ((hid <= 0) && (hid > int(m_heads.size() + 1)))
  {
    return -1;
  }
  return m_heads[hid - 1]->update(evtname);
}

//
// Carrier ID Reader/Writer Head
//
CIDRWHead::CIDRWHead(const BCHAR * idp) : bee::state_task("HEAD"),
  m_headStatus(NO_OPERATING), m_id(idp), m_cycles(0), m_condition(NO_ALARMS),
  m_installed(), m_maintenanceData(_TX(""))
{
  m_uniqueName += m_id;  // append head-id
  CIDRWHeadState * hds = new CIDRWHeadState(this);
  m_status = hds;
}

CIDRWHead::~CIDRWHead()
{
  CIDRWHeadState * hds = static_cast<CIDRWHeadState *>(m_status);
  hds->terminate();
  delete hds;
}

#if 0
void CIDRWHead::copy(const CIDRWHead& rhs)
{
  this->bee::state_task::operator =(rhs);

  m_status = rhs.m_status;
  m_id = rhs.m_id;
  m_cycles = rhs.m_cycles;
  m_installed = rhs.m_installed;
  m_maintenanceData = rhs.m_maintenanceData;
}
#endif

int CIDRWHead::open0()
{
  int result = this->open(null);
  this->on_trigger_connect(boost::bind(&gem::event_manager::notify, &evtmngr, _1, _2));

  CIDRWHeadState * hds = static_cast<CIDRWHeadState *>(m_status);
  hds->initiate();
  return result;
}

const BCHAR * CIDRWHead::current_state()
{
  CIDRWHeadState * hds = static_cast<CIDRWHeadState *>(m_status);
  return hds->current_state();
}

} }  // *namespace* jyugem::cidrw
