// $Id$

//=============================================================================
/**
 *  @file    Carrier.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_CSM_CARRIER_H
#define JYUGEM_CSM_CARRIER_H

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif


#include <boost/mpl/list.hpp>
#include <boost/format.hpp>

#include "jyugem/bee/state_task.h"
namespace bee = jyugem::bee;


//
namespace jyugem { namespace csm
{

//
// E87 : SPECIFICATION FOR CARRIER MANAGEMENT
//

class Content
{
public:
    Content() {}
    Content(const std::string& lotid, const std::string& substid)
        : m_lotID(lotid), m_substID(substid) {}
    Content(const Content& rhs)
        : m_lotID(rhs.m_lotID), m_substID(rhs.m_substID) {}
    ~Content() {}

    Content& operator=(const Content& rhs)
    {
        if (this == &rhs)
           return *this;
        m_lotID = rhs.m_lotID;
        m_substID = rhs.m_substID;
        return *this;
    }

    std::string LotID()       { return m_lotID; }
    std::string SubstrateID() { return m_substID; }

    std::string m_lotID;
    std::string m_substID;
};


//------------------------------------------------------------------------------
// State of Carrier
//------------------------------------------------------------------------------
class Carrier : public bee::state_task
{
    friend class CarrierState;

public:
    Carrier();
//  Carrier(const Carrier& carrier);
    virtual ~Carrier() {}

//  Carrier& operator=(const Carrier& rhs)
//  {
//      if ( this== &rhs)
//          return *this;
//      this->copy(rhs);
//      return *this;
//  }

    enum            // CarrierID Status
    {
        ID_NOT_READ,
        ID_WAITING_FOR_HOST,
        ID_VERIFICATION_OK,
        ID_VERIFICATION_FAILED
    };

    enum            // CarrierAccessing Status
    {
        NOT_ACCESSED,
        IN_ACCESS,
        CARRIER_COMPLETE,
        CARRIER_STOPPED
    };

    enum            // SlotMap Status
    {
        SLOTMAP_NOT_READ,
        SLOT_WAITING_FOR_HOST,
        SLOTMAP_VERIFICATION_OK,
        SLOTMAP_VERIFICATION_FAILED,
    };

protected:
//  void copy(const Carrier& rhs);

protected:
    bool m_bypassReadID;

    // Attribute data pointer in Object
    size_t m_capacity;
    size_t m_substrateCount;
    int    m_carrierIDStatus;
    int    m_carrierAccessingStatus;
    int    m_slotMapStatus;
    std::string m_locationID;
    std::string m_usage;
    vector<Content *> m_contentMap;
    vector<Slot *>    m_slotMap;
    //
};


#endif  // JYUGEM_CSM_CARRIER_H
